@@ -4,6 +4,7 @@
 
 typedef struct FFPackagesResult
 {
+    uint32_t am;
     uint32_t apk;
     uint32_t brew;
     uint32_t brewCask;
@@ -13,6 +14,8 @@ typedef struct FFPackagesResult
     uint32_t eopkg;
     uint32_t flatpakSystem;
     uint32_t flatpakUser;
+    uint32_t lpkg;
+    uint32_t lpkgbuild;
     uint32_t nixDefault;
     uint32_t nixSystem;
     uint32_t nixUser;
@@ -25,6 +28,7 @@ typedef struct FFPackagesResult
     uint32_t rpm;
     uint32_t scoop;
     uint32_t snap;
+    uint32_t sorcery;
     uint32_t winget;
     uint32_t xbps;
 
