@@ -7,6 +7,7 @@
 #include "modules/bluetooth/bluetooth.h"
 #include "modules/brightness/brightness.h"
 #include "modules/board/board.h"
+#include "modules/bootmgr/bootmgr.h"
 #include "modules/break/break.h"
 #include "modules/camera/camera.h"
 #include "modules/chassis/chassis.h"
@@ -28,6 +29,7 @@
 #include "modules/icons/icons.h"
 #include "modules/kernel/kernel.h"
 #include "modules/lm/lm.h"
+#include "modules/loadavg/loadavg.h"
 #include "modules/locale/locale.h"
 #include "modules/localip/localip.h"
 #include "modules/media/media.h"
@@ -39,6 +41,7 @@
 #include "modules/os/os.h"
 #include "modules/packages/packages.h"
 #include "modules/physicaldisk/physicaldisk.h"
+#include "modules/physicalmemory/physicalmemory.h"
 #include "modules/player/player.h"
 #include "modules/poweradapter/poweradapter.h"
 #include "modules/processes/processes.h"
