@@ -7,6 +7,7 @@ void ffOptionsInitModules(FFOptionsModules* options)
     ffInitBiosOptions(&options->bios);
     ffInitBluetoothOptions(&options->bluetooth);
     ffInitBoardOptions(&options->board);
+    ffInitBootmgrOptions(&options->bootmgr);
     ffInitBreakOptions(&options->break_);
     ffInitBrightnessOptions(&options->brightness);
     ffInitCameraOptions(&options->camera);
@@ -73,6 +74,7 @@ void ffOptionsDestroyModules(FFOptionsModules* options)
     ffDestroyBiosOptions(&options->bios);
     ffDestroyBluetoothOptions(&options->bluetooth);
     ffDestroyBoardOptions(&options->board);
+    ffDestroyBootmgrOptions(&options->bootmgr);
     ffDestroyBreakOptions(&options->break_);
     ffDestroyBrightnessOptions(&options->brightness);
     ffDestroyCameraOptions(&options->camera);
